@@ -30,7 +30,7 @@ enum {
 
 /* Tux filenames: */
 
-char * tux_img_fnames[NUM_TIP_TUX] = {
+const char * const tux_img_fnames[NUM_TIP_TUX] = {
   DATA_PREFIX "images/tux/default.png",
   DATA_PREFIX "images/tux/kiss.png",
   DATA_PREFIX "images/tux/bored.png",
