@@ -15,8 +15,11 @@
 
 /* if printcfg is NULL, uses the default printer */
 extern const char *SurfacePrint( SDL_Surface *surf, 
-			                     const char *printcfg, 
-			                     int showdialog );
+			         const char *printcfg, 
+			         int showdialog );
 extern int IsPrinterAvailable( void );
 
+/* additional windows functions requiring <windows.h> */
+extern char *GetDefaultSaveDir(const char *suffix);
+
 #endif
